// 
// Copyright (C) 2009, Golden Rule Trading, LLC
// Golden Rule Trading, LLC reserves the right to modify or overwrite this NinjaScript.
//
// NOTE:
// Portion within #region Miscellaneous is
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	[Description("Multiple Pivot point calculator for Classic, Camarilla, && Woodie Formulas. I use mypivots.com for prior day's OHLC futures data")]
	public class GRTPivotsMulti : Indicator
	{
		#region Variables
			private double			userDefinedPOpen	= 0;
			private double			userDefinedTOpen	= 0;
			private double			userDefinedHigh		= 0;
			private double			userDefinedLow		= 0;
			private double			userDefinedClose	= 0;
			private int				width			= 93;
			private bool			showUserPOpen	= true;
			private bool			showUserTOpen	= true;
			private bool			showUserHigh	= true;
			private bool			showUserLow		= true;
			private bool			showUserClose	= true;
			private bool			showWoodie		= true;
			private bool			showCamarilla	= true;
			private bool			showClassic		= true;
			private	double	ppW = 0,				ppCl	=0;
			private	double	r1W	= 0,	r1C	= 0,	r1Cl	= 0;
			private	double	r2W	= 0,	r2C	= 0,	r2Cl	= 0;
			private	double	r3W	= 0,	r3C	= 0,	r3Cl	= 0;
			private	double	r4W	= 0,	r4C	= 0,	r4Cl	= 0;
			private	double	s1W	= 0,	s1C	= 0,	s1Cl	= 0;
			private	double	s2W	= 0,	s2C	= 0,	s2Cl	= 0;
			private	double	s3W	= 0,	s3C	= 0,	s3Cl	= 0;
			private	double	s4W	= 0,	s4C	= 0,	s4Cl	= 0;
			private	StringFormat	stringFormatFar			= new StringFormat();
			private	StringFormat	stringFormatNear		= new StringFormat();
			private	SolidBrush[]	brushes					= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
																new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
																new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
																new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
																new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
																new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
																new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
																new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black)};
			private DateTime			currentDate			= Cbi.Globals.MinDate;
			private DateTime			currentMonth		= Cbi.Globals.MinDate;
			private DateTime			currentWeek			= Cbi.Globals.MinDate;
			private Bars				dailyBars			= null;
			private	string				errorData			= "Insufficient historical data to calculate pivots. Increase chart look back period (days back).";
			private float				errorTextWidth		= 0;
			private float				errorTextHeight		= 0;
			private bool				existsHistDailyData	= false;
			private bool				isDailyDataLoaded	= false;
			private bool				isInitializing		= false;
			private Data.PivotRange		pivotRangeType		= PivotRange.Daily;
			private HLCCalculationMode	priorDayHLC			= HLCCalculationMode.DailyBars;
			private SolidBrush			textBrush			= new SolidBrush(Color.Black);
			private Font				textFont			= new Font("Arial", 30);
			private WoodieFormula		woodieChose			= WoodieFormula.CurrentOpen;
		#endregion

		protected override void Initialize()
		{
			Add(new Plot(Color.DarkGreen,	"Prior High"));							// [0]
			Add(new Plot(Color.DarkRed,		"Prior Low"));							// [1]
			Add(new Plot(Color.DarkOrange,	"Prior Close"));						// [2]
			Add(new Plot(Color.DarkCyan,	"Today Open"));							// [3]
			Add(new Plot(Color.DarkCyan,	"Prior Open"));							// [4]
				Plots[0].Pen.DashStyle = DashStyle.DashDot;	Plots[0].Pen.Width = 2;
				Plots[1].Pen.DashStyle = DashStyle.DashDot;	Plots[1].Pen.Width = 2;
				Plots[2].Pen.DashStyle = DashStyle.DashDot;	Plots[2].Pen.Width = 2;
				Plots[3].Pen.DashStyle = DashStyle.DashDot;	Plots[3].Pen.Width = 2;
				Plots[4].Pen.DashStyle = DashStyle.DashDot;	Plots[4].Pen.Width = 2;
			
			Add(new Plot(Color.DarkBlue,	"PPW"));	Plots[5].Pen.Width = 2;		// [5]		If you Add a new Plot, 
			Add(new Plot(Color.RoyalBlue,	"R1W"));	Plots[6].Pen.Width = 2;		// [6]		you Must also add a
			Add(new Plot(Color.DeepPink,	"S1W"));	Plots[7].Pen.Width = 2;		// [7]		new SolidBrush in the
			Add(new Plot(Color.RoyalBlue,	"R2W"));	Plots[8].Pen.Width = 2;		// [8]		Variables section
			Add(new Plot(Color.DeepPink,	"S2W"));	Plots[9].Pen.Width = 2;		// [9]
			Add(new Plot(Color.Blue,		"R3W"));	Plots[10].Pen.Width = 2;	// [10]
			Add(new Plot(Color.Crimson,		"S3W"));	Plots[11].Pen.Width = 2;	// [11]
			Add(new Plot(Color.Blue,		"R4W"));	Plots[12].Pen.Width = 3;	// [12]
			Add(new Plot(Color.Crimson,		"S4W"));	Plots[13].Pen.Width = 3;	// [13]
			
			Add(new Plot(Color.DarkBlue,		"PPCL"));	Plots[14].Pen.Width = 2;	// [14]

			Add(new Plot(Color.LavenderBlush,	"R1Cam"));	Plots[15].Pen.Width = 1;	// [15]
			Add(new Plot(Color.LightCyan,		"S1Cam"));	Plots[16].Pen.Width = 1;	// [16]
			Add(new Plot(Color.LavenderBlush,	"R2Cam"));	Plots[17].Pen.Width = 1;	// [17]
			Add(new Plot(Color.LightCyan,		"S2Cam"));	Plots[18].Pen.Width = 1;	// [18]
			Add(new Plot(Color.Blue,			"R3Cam"));	Plots[19].Pen.Width = 2;	// [19]
			Add(new Plot(Color.Crimson,			"S3Cam"));	Plots[20].Pen.Width = 2;	// [20]
			Add(new Plot(Color.Blue,			"R4Cam"));	Plots[21].Pen.Width = 3;	// [21]
			Add(new Plot(Color.Crimson,			"S4Cam"));	Plots[22].Pen.Width = 3;	// [22]
			
			Add(new Plot(Color.RoyalBlue,		"R1CL"));	Plots[23].Pen.Width = 2;	// [23]
			Add(new Plot(Color.DeepPink,		"S1CL"));	Plots[24].Pen.Width = 2;	// [24]
			Add(new Plot(Color.RoyalBlue,		"R2CL"));	Plots[25].Pen.Width = 2;	// [25]
			Add(new Plot(Color.DeepPink,		"S2CL"));	Plots[26].Pen.Width = 2;	// [26]
			Add(new Plot(Color.Blue,			"R3CL"));	Plots[27].Pen.Width = 2;	// [27]
			Add(new Plot(Color.Crimson,			"S3CL"));	Plots[28].Pen.Width = 2;	// [28]
			Add(new Plot(Color.Blue,			"R4CL"));	Plots[29].Pen.Width = 3;	// [29]
			Add(new Plot(Color.Crimson,			"S4CL"));	Plots[30].Pen.Width = 3;	// [30]

			stringFormatFar.Alignment	= StringAlignment.Far;

			CalculateOnBarClose	= true;
			ChartOnly			= true;
            AutoScale			= false;
			DisplayInDataBox	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			PaintPriceMarkers	= false;
		}

		protected override void OnBarUpdate()
		{
			if (Bars == null)		return; 
			if (ppCl == 0)
			{
				ppCl			= (userDefinedHigh + userDefinedLow + userDefinedClose) / 3;
				double range	= userDefinedHigh - userDefinedLow;
				if (showWoodie)
				{
					if (woodieChose == WoodieFormula.CurrentOpen)
						ppW	= (userDefinedHigh + userDefinedLow + 2*userDefinedTOpen) / 4;
					else
						ppW	= (userDefinedHigh + userDefinedLow + 2*userDefinedClose) / 4;
					s1W				= 2 * ppW - userDefinedHigh;// S1 = (2 * PP) - HIGH
					r1W				= 2 * ppW - userDefinedLow;	// R1 = (2 * PP) - LOW
					s2W				= ppW - range;				// S2 = PP - RANGE
					r2W				= ppW + range;				// R2 = PP + RANGE
					s3W				= s1W - range;				// S3 = L - 2 * (H - PP) (same as: S1 - RANGE)
					r3W				= r1W + range;				// R3 = H + 2 * (PP - L) (same as: R1 + RANGE)
					s4W				= s3W - range;				// S4 = S3 - RANGE
					r4W				= r3W + range;				// R4 = R3 + RANGE
				}
				if (showCamarilla)
				{
					s1C				= userDefinedClose - range * 1.1/12;	// S1 = C - RANGE * 1.1/12
					r1C				= userDefinedClose + range * 1.1/12;	// R1 = C + RANGE * 1.1/12
					s2C				= userDefinedClose - range * 1.1/6;		// S2 = C - RANGE * 1.1/6
					r2C				= userDefinedClose + range * 1.1/6;		// R2 = C + RANGE * 1.1/6
					s3C				= userDefinedClose - range * 1.1/4;		// S3 = C - RANGE * 1.1/4
					r3C				= userDefinedClose + range * 1.1/4;		// R3 = C + RANGE * 1.1/4
					s4C				= userDefinedClose - range * 1.1/2;		// S4 = C - RANGE * 1.1/2 
					r4C				= userDefinedClose + range * 1.1/2;		// R4 = C + RANGE * 1.1/2
				}
				if (showClassic)
				{
					s1Cl			= 2 * ppCl - userDefinedHigh;	// S1 = (2 * PP) - HIGH
					r1Cl			= 2 * ppCl - userDefinedLow;	// R1 = (2 * PP) - LOW
					s2Cl			= ppCl - range;				// S2 = PP - RANGE
					r2Cl			= ppCl + range;				// R2 = PP + RANGE
					s3Cl			= s2Cl - range;				// S3 = S2 - RANGE (same as: PP - RANGE * 2)
					r3Cl			= r2Cl + range;				// R3 = R2 + RANGE (same as: PP + RANGE * 2)
					s4Cl			= s3Cl - range;				// S4 = S3 - RANGE (same as: PP - RANGE * 3)
					r4Cl			= r3Cl + range;				// R4 = R3 + RANGE (same as: PP + RANGE * 3)
				}
			}
			if (showWoodie)
			{	PPW.Set(ppW);
				R1W.Set(r1W);	S1W.Set(s1W);
				R2W.Set(r2W);	S2W.Set(s2W);
				R3W.Set(r3W);	S3W.Set(s3W);
				R4W.Set(r4W);	S4W.Set(s4W);
			}
			if (showCamarilla || showClassic) PPCL.Set(ppCl);
			if (showCamarilla)
			{
				R1C.Set(r1C);	S1C.Set(s1C);
				R2C.Set(r2C);	S2C.Set(s2C);
				R3C.Set(r3C);	S3C.Set(s3C);
				R4C.Set(r4C);	S4C.Set(s4C);
			}
			if (showClassic)
			{	
				R1CL.Set(r1Cl);	S1CL.Set(s1Cl);
				R2CL.Set(r2Cl);	S2CL.Set(s2Cl);
				R3CL.Set(r3Cl);	S3CL.Set(s3Cl);
				R4CL.Set(r4Cl);	S4CL.Set(s4Cl);
			}
			if (showUserTOpen && userDefinedTOpen != 0)	TodayOpen.Set(userDefinedTOpen);
			if (showUserPOpen && userDefinedPOpen != 0)	PriorOpen.Set(userDefinedPOpen);
			if (showUserHigh  && userDefinedHigh != 0)	PriorHigh.Set(userDefinedHigh);
			if (showUserLow   && userDefinedLow != 0)	PriorLow.Set(userDefinedLow);
			if (showUserClose && userDefinedClose != 0)	PriorClose.Set(userDefinedClose);
		}

		#region Properties
		[Description("Today's session Open value. Only Needed for Woodie's original formula.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("0.Todays Open")]
		public double UserDefinedTOpen
		{
			get { return userDefinedTOpen; }
			set { userDefinedTOpen = value; }
		}

		[Description("Prior session Open value. Not needed for pivot calculation.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("1.Prior Open")]
		public double UserDefinedPOpen
		{
			get { return userDefinedPOpen; }
			set { userDefinedPOpen = value; }
		}

		[Description("Prior session High value used for pivots calculation.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("2.Prior High")]
		public double UserDefinedHigh
		{
			get { return userDefinedHigh; }
			set { userDefinedHigh = value; }
		}

		[Description("Prior session Low value used for pivots calculation.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("3.Prior Low")]
		public double UserDefinedLow
		{
			get { return userDefinedLow; }
			set { userDefinedLow = value; }
		}

		[Description("Prior session Close value used for pivots calculation.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("4.Prior Close")]
		public double UserDefinedClose
		{
			get { return userDefinedClose; }
			set { userDefinedClose = value; }
		}

		[Description("Show Camarilla pivot points.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Camarilla")]
		public bool ShowCamarilla
		{
			get { return showCamarilla; }
			set { showCamarilla = value; }
		}

		[Description("Show Classic pivot points.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Classic")]
		public bool ShowClassic
		{
			get { return showClassic; }
			set { showClassic = value; }
		}

		[Description("Show Woodie pivot points.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Woodie")]
		public bool ShowWoodie
		{
			get { return showWoodie; }
			set { showWoodie = value; }
		}

		[Description("Plot today's Open line.")]
		[Gui.Design.DisplayNameAttribute("0.Show Todays Open")]
        public bool ShowUserTOpen
        {
            get { return showUserTOpen; }
			set { showUserTOpen = value; }
        }

		[Description("Plot prior day's Open line.")]
		[Gui.Design.DisplayNameAttribute("1.Show Prior Open")]
        public bool ShowUserPOpen
        {
            get { return showUserPOpen; }
			set { showUserPOpen = value; }
        }

		[Description("Plot prior day's High line.")]
		[Gui.Design.DisplayNameAttribute("2.Show prior High")]
        public bool ShowUserHigh
        {
            get { return showUserHigh; }
			set { showUserHigh = value; }
        }

		[Description("Plot prior day's Low line.")]
		[Gui.Design.DisplayNameAttribute("3.Show prior Low")]
        public bool ShowUserLow
        {
            get { return showUserLow; }
			set { showUserLow = value; }
        }

		[Description("Plot prior day's Close line.")]
		[Gui.Design.DisplayNameAttribute("4.Show prior Close")]
        public bool ShowUserClose
        {
            get { return showUserClose; }
			set { showUserClose = value; }
        }

		[Description("Length of the pivot lines as # of bars.")]
		[Category("Parameters")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(1, value); }
		}
		
		[Category("Parameters"), Description("Choose which Woodies formula to use. CurrentOpen = (original)PP=(priorHigh + priorLow + 2*CurrentOPEN)/4. PriorClose = (modified)PP=(priorHigh + priorLow + 2*PriorCLOSE)/4")]
		[Gui.Design.DisplayNameAttribute("Woodie Formula")]
		public WoodieFormula WoodieChose
		{	get	{return woodieChose;	}
			set	{woodieChose = value;	}
		}
// ******* PLOTS *****************************************************************
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
		[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public DataSeries PriorHigh
		{
			get { return Values[0]; }
		}
		
		[Browsable(false)]	[XmlIgnore()]
		public DataSeries PriorLow
		{
			get { return Values[1]; }
		}
		
		[Browsable(false)]	[XmlIgnore()]
		public DataSeries PriorClose
		{
			get { return Values[2]; }
		}
		
		[Browsable(false)]	[XmlIgnore()]
		public DataSeries TodayOpen
		{
			get { return Values[3]; }
		}
		
		[Browsable(false)]	[XmlIgnore()]
		public DataSeries PriorOpen
		{
			get { return Values[4]; }
		}
// ******* WOODIE **********
		[Browsable(false)]	[XmlIgnore()]
		public DataSeries PPW
		{
			get { return Values[5]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R1W
		{
			get { return Values[6]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S1W
		{
			get { return Values[7]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R2W
		{
			get { return Values[8]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S2W
		{
			get { return Values[9]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R3W
		{
			get { return Values[10]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S3W
		{
			get { return Values[11]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R4W
		{
			get { return Values[12]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S4W
		{
			get { return Values[13]; }
		}
// ******** PP Cl & Cam *********
		[Browsable(false)]	[XmlIgnore()]
		public DataSeries PPCL
		{
			get { return Values[14]; }
		}
// ******** CAMARILLA *********
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R1C
		{
			get { return Values[15]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S1C
		{
			get { return Values[16]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R2C
		{
			get { return Values[17]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S2C
		{
			get { return Values[18]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R3C
		{
			get { return Values[19]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S3C
		{
			get { return Values[20]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R4C
		{
			get { return Values[21]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S4C
		{
			get { return Values[22]; }
		}
// ******** CLASSIC *********
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R1CL
		{
			get { return Values[23]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S1CL
		{
			get { return Values[24]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R2CL
		{
			get { return Values[25]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S2CL
		{
			get { return Values[26]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R3CL
		{
			get { return Values[27]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S3CL
		{
			get { return Values[28]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries R4CL
		{
			get { return Values[29]; }
		}
		
		[Browsable(false)]	[XmlIgnore]
		public DataSeries S4CL
		{
			get { return Values[30]; }
		}
		#endregion

		#region Miscellaneous
		public enum WoodieFormula
		{
			CurrentOpen,
			PriorClose,
		}

		private int GetDayOfWeek(DateTime date)
		{
			DateTime saturday = new DateTime(1776, 7, 4).AddDays(2);
			TimeSpan diff = date.Subtract(saturday);
			return (diff.Days % 7);
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6-GetDayOfWeek(time)));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}

		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;

			// plot error if data not complete
			if (textBrush.Color != ChartControl.AxisColor || textFont != ChartControl.Font)
			{
				textBrush.Color = ChartControl.AxisColor;
                textFont = ChartControl.Font;

				SizeF errorSize	= graphics.MeasureString(errorData, textFont);
				errorTextWidth	= errorSize.Width + 5;
				errorTextHeight	= errorSize.Height + 5;
			}

			if (priorDayHLC == HLCCalculationMode.CalcFromIntradayData) 
			{
				DateTime lastBarDate = Bars.Get(Math.Min(Bars.Count - 1, ChartControl.LastBarPainted)).Time.Date;
				if ((Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday
						&& ((pivotRangeType == PivotRange.Monthly && Bars.Count > 0 && Bars.Get(0).Time.AddSeconds(-1).Date >= new DateTime(lastBarDate.Year, lastBarDate.Month, 1).AddMonths(-1)) 
							|| (pivotRangeType == PivotRange.Weekly && Bars.Count > 0 && Bars.Get(0).Time.AddSeconds(-1).Date >= lastBarDate.AddDays(-GetDayOfWeek(lastBarDate)).AddDays(-7))
							|| (pivotRangeType == PivotRange.Daily && Bars.Count > 0 && Bars.Get(0).Time.AddSeconds(-1).Date >= lastBarDate.AddDays(-1))))
					|| (!Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday
						&& ((pivotRangeType == PivotRange.Monthly && Bars.Count > 0 && Bars.Get(0).Time.Date >= new DateTime(lastBarDate.Year, lastBarDate.Month, 1).AddMonths(-1)) 
							|| (pivotRangeType == PivotRange.Weekly && Bars.Count > 0 && Bars.Get(0).Time.Date >= lastBarDate.AddDays(-GetDayOfWeek(lastBarDate)).AddDays(-7))
							|| (pivotRangeType == PivotRange.Daily && Bars.Count > 0 && Bars.Get(0).Time.Date >= lastBarDate.AddDays(-1)))))
				{
					graphics.DrawString(errorData, ChartControl.Font, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);
				}
			}
			else if (priorDayHLC == HLCCalculationMode.DailyBars && existsHistDailyData)
			{
				DateTime lastBarDate = dailyBars.Get(Math.Min(dailyBars.Count - 1, ChartControl.LastBarPainted)).Time.Date;
				if ((pivotRangeType == PivotRange.Monthly && dailyBars.Get(0).Time.Date >= new DateTime(lastBarDate.Year, lastBarDate.Month, 1).AddMonths(-1)) 
					|| (pivotRangeType == PivotRange.Weekly && dailyBars.Get(0).Time.Date >= lastBarDate.AddDays(-GetDayOfWeek(lastBarDate)).AddDays(-7))
					|| (pivotRangeType == PivotRange.Daily && dailyBars.Get(0).Time.Date >= lastBarDate.AddDays(-1)))
				{
					graphics.DrawString(errorData, ChartControl.Font, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);
				}
			} 

			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				DateTime		lastDate			= Cbi.Globals.MinDate;
				DateTime		lastWeek			= Cbi.Globals.MinDate;
				DateTime		lastMonth			= Cbi.Globals.MinDate;
				int				lastX				= -1;
				int				lastY				= -1;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				GraphicsPath	path				= new GraphicsPath();
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries	series				= (DataSeries) Values[seriesCount];

				if (brush.Color != plot.Pen.Color)	
					brush = new SolidBrush(plot.Pen.Color);

				for (int count = ChartControl.BarsPainted - 1; count >= ChartControl.BarsPainted - Math.Min(ChartControl.BarsPainted, Width); count--)
				{
					int whiteBars = Math.Max(0, ChartControl.LastBarPainted - ChartControl.Bars[0].Count + 1);
					int idx = ChartControl.LastBarPainted - whiteBars - ChartControl.BarsPainted + 1 + count;
					if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
						continue;
					else if (!series.IsValidPlot(idx))
						continue;

					if (pivotRangeType == PivotRange.Daily && lastDate == Cbi.Globals.MinDate) 
						lastDate = Bars.Get(idx).Time.AddSeconds(-1).Date;
					else if (pivotRangeType == PivotRange.Weekly && lastWeek == Cbi.Globals.MinDate) 
						lastWeek = RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Weekly);
					else if (pivotRangeType == PivotRange.Monthly && lastMonth == Cbi.Globals.MinDate) 
						lastMonth = RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Monthly);
					else if (pivotRangeType == PivotRange.Daily && lastDate != Bars.Get(idx).Time.Date 
							|| pivotRangeType == PivotRange.Weekly && lastWeek != RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Weekly)
							|| pivotRangeType == PivotRange.Monthly && lastMonth != RoundUpTimeToPeriodTime(Bars.Get(idx).Time.Date, PivotRange.Monthly))
						continue;

					double		val = series.Get(idx);
					int			x	= (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2
										- (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + (count - whiteBars) * ChartControl.BarSpace);
					int			y	= (int) ((bounds.Y + bounds.Height) - ((val - min ) / Gui.Chart.ChartControl.MaxMinusMin(max, min)) * bounds.Height);

					if (lastX >= 0)
					{
						if (y != lastY) // Problem here is, that last bar of old day has date of new day
							y = lastY;
						path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
					}
					lastX	= x;
					lastY	= y;
				}

				graphics.SmoothingMode = SmoothingMode.AntiAlias;
				graphics.DrawPath(plot.Pen, path);
				graphics.SmoothingMode = oldSmoothingMode;
				graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX, lastY - ChartControl.Font.GetHeight() / 2, stringFormatFar);
			}
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private GRTPivotsMulti[] cacheGRTPivotsMulti = null;

        private static GRTPivotsMulti checkGRTPivotsMulti = new GRTPivotsMulti();

        /// <summary>
        /// Multiple Pivot point calculator for Classic, Camarilla, && Woodie Formulas. I use mypivots.com for prior day's OHLC futures data
        /// </summary>
        /// <returns></returns>
        public GRTPivotsMulti GRTPivotsMulti(bool showCamarilla, bool showClassic, bool showWoodie, double userDefinedClose, double userDefinedHigh, double userDefinedLow, double userDefinedPOpen, double userDefinedTOpen, int width)
        {
            return GRTPivotsMulti(Input, showCamarilla, showClassic, showWoodie, userDefinedClose, userDefinedHigh, userDefinedLow, userDefinedPOpen, userDefinedTOpen, width);
        }

        /// <summary>
        /// Multiple Pivot point calculator for Classic, Camarilla, && Woodie Formulas. I use mypivots.com for prior day's OHLC futures data
        /// </summary>
        /// <returns></returns>
        public GRTPivotsMulti GRTPivotsMulti(Data.IDataSeries input, bool showCamarilla, bool showClassic, bool showWoodie, double userDefinedClose, double userDefinedHigh, double userDefinedLow, double userDefinedPOpen, double userDefinedTOpen, int width)
        {
            checkGRTPivotsMulti.ShowCamarilla = showCamarilla;
            showCamarilla = checkGRTPivotsMulti.ShowCamarilla;
            checkGRTPivotsMulti.ShowClassic = showClassic;
            showClassic = checkGRTPivotsMulti.ShowClassic;
            checkGRTPivotsMulti.ShowWoodie = showWoodie;
            showWoodie = checkGRTPivotsMulti.ShowWoodie;
            checkGRTPivotsMulti.UserDefinedClose = userDefinedClose;
            userDefinedClose = checkGRTPivotsMulti.UserDefinedClose;
            checkGRTPivotsMulti.UserDefinedHigh = userDefinedHigh;
            userDefinedHigh = checkGRTPivotsMulti.UserDefinedHigh;
            checkGRTPivotsMulti.UserDefinedLow = userDefinedLow;
            userDefinedLow = checkGRTPivotsMulti.UserDefinedLow;
            checkGRTPivotsMulti.UserDefinedPOpen = userDefinedPOpen;
            userDefinedPOpen = checkGRTPivotsMulti.UserDefinedPOpen;
            checkGRTPivotsMulti.UserDefinedTOpen = userDefinedTOpen;
            userDefinedTOpen = checkGRTPivotsMulti.UserDefinedTOpen;
            checkGRTPivotsMulti.Width = width;
            width = checkGRTPivotsMulti.Width;

            if (cacheGRTPivotsMulti != null)
                for (int idx = 0; idx < cacheGRTPivotsMulti.Length; idx++)
                    if (cacheGRTPivotsMulti[idx].ShowCamarilla == showCamarilla && cacheGRTPivotsMulti[idx].ShowClassic == showClassic && cacheGRTPivotsMulti[idx].ShowWoodie == showWoodie && Math.Abs(cacheGRTPivotsMulti[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cacheGRTPivotsMulti[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cacheGRTPivotsMulti[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && Math.Abs(cacheGRTPivotsMulti[idx].UserDefinedPOpen - userDefinedPOpen) <= double.Epsilon && Math.Abs(cacheGRTPivotsMulti[idx].UserDefinedTOpen - userDefinedTOpen) <= double.Epsilon && cacheGRTPivotsMulti[idx].Width == width && cacheGRTPivotsMulti[idx].EqualsInput(input))
                        return cacheGRTPivotsMulti[idx];

            GRTPivotsMulti indicator = new GRTPivotsMulti();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.ShowCamarilla = showCamarilla;
            indicator.ShowClassic = showClassic;
            indicator.ShowWoodie = showWoodie;
            indicator.UserDefinedClose = userDefinedClose;
            indicator.UserDefinedHigh = userDefinedHigh;
            indicator.UserDefinedLow = userDefinedLow;
            indicator.UserDefinedPOpen = userDefinedPOpen;
            indicator.UserDefinedTOpen = userDefinedTOpen;
            indicator.Width = width;
            indicator.SetUp();

            GRTPivotsMulti[] tmp = new GRTPivotsMulti[cacheGRTPivotsMulti == null ? 1 : cacheGRTPivotsMulti.Length + 1];
            if (cacheGRTPivotsMulti != null)
                cacheGRTPivotsMulti.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheGRTPivotsMulti = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Multiple Pivot point calculator for Classic, Camarilla, && Woodie Formulas. I use mypivots.com for prior day's OHLC futures data
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.GRTPivotsMulti GRTPivotsMulti(bool showCamarilla, bool showClassic, bool showWoodie, double userDefinedClose, double userDefinedHigh, double userDefinedLow, double userDefinedPOpen, double userDefinedTOpen, int width)
        {
            return _indicator.GRTPivotsMulti(Input, showCamarilla, showClassic, showWoodie, userDefinedClose, userDefinedHigh, userDefinedLow, userDefinedPOpen, userDefinedTOpen, width);
        }

        /// <summary>
        /// Multiple Pivot point calculator for Classic, Camarilla, && Woodie Formulas. I use mypivots.com for prior day's OHLC futures data
        /// </summary>
        /// <returns></returns>
        public Indicator.GRTPivotsMulti GRTPivotsMulti(Data.IDataSeries input, bool showCamarilla, bool showClassic, bool showWoodie, double userDefinedClose, double userDefinedHigh, double userDefinedLow, double userDefinedPOpen, double userDefinedTOpen, int width)
        {
            return _indicator.GRTPivotsMulti(input, showCamarilla, showClassic, showWoodie, userDefinedClose, userDefinedHigh, userDefinedLow, userDefinedPOpen, userDefinedTOpen, width);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Multiple Pivot point calculator for Classic, Camarilla, && Woodie Formulas. I use mypivots.com for prior day's OHLC futures data
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.GRTPivotsMulti GRTPivotsMulti(bool showCamarilla, bool showClassic, bool showWoodie, double userDefinedClose, double userDefinedHigh, double userDefinedLow, double userDefinedPOpen, double userDefinedTOpen, int width)
        {
            return _indicator.GRTPivotsMulti(Input, showCamarilla, showClassic, showWoodie, userDefinedClose, userDefinedHigh, userDefinedLow, userDefinedPOpen, userDefinedTOpen, width);
        }

        /// <summary>
        /// Multiple Pivot point calculator for Classic, Camarilla, && Woodie Formulas. I use mypivots.com for prior day's OHLC futures data
        /// </summary>
        /// <returns></returns>
        public Indicator.GRTPivotsMulti GRTPivotsMulti(Data.IDataSeries input, bool showCamarilla, bool showClassic, bool showWoodie, double userDefinedClose, double userDefinedHigh, double userDefinedLow, double userDefinedPOpen, double userDefinedTOpen, int width)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.GRTPivotsMulti(input, showCamarilla, showClassic, showWoodie, userDefinedClose, userDefinedHigh, userDefinedLow, userDefinedPOpen, userDefinedTOpen, width);
        }

    }
}
#endregion
